#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Tracing module."""

from pathlib import Path
from typing import Sequence, Union

from biolib.pyppeteer.pyppeteer import helpers
from biolib.pyppeteer.pyppeteer.connection import CDPSession


class Tracing:
    """Tracing class.

    You can use :meth:`start` and :meth:`stop` to create a trace file which can
    be opened in Chrome DevTools or
    `timeline viewer <https://chromedevtools.github.io/timeline-viewer/>`_.

    .. code::

        await page.tracing.start({'path': 'trace.json'})
        await page.goto('https://www.google.com')
        await page.tracing.stop()
    """

    def __init__(self, client: CDPSession) -> None:
        self._client = client
        self._recording = False
        self._path = ''

    async def start(
        self, path: Union[Path, str] = '', screenshots: bool = False, categories: Sequence[str] = None
    ) -> None:
        """Start tracing.

        Only one trace can be active at a time per browser.

        This method accepts the following options:

        * ``path`` (str): A path to write the trace file to.
        * ``screenshots`` (bool): Capture screenshots in the trace.
        * ``categories`` (List[str]): Specify custom categories to use instead
          of default.
        """
        defaultCategories = [
            '-*',
            'devtools.timeline',
            'v8.execute',
            'disabled-by-default-devtools.timeline',
            'disabled-by-default-devtools.timeline.frame',
            'toplevel',
            'blink.console',
            'blink.user_timing',
            'latencyInfo',
            'disabled-by-default-devtools.timeline.stack',
            'disabled-by-default-v8.cpu_profiler',
            'disabled-by-default-v8.cpu_profiler.hires',
        ]

        if not isinstance(categories, list):
            try:
                categories = list(categories)
            except TypeError:
                categories = defaultCategories

        if screenshots:
            categories.append('disabled-by-default-devtools.screenshot')

        self._path = path
        self._recording = True
        await self._client.send(
            'Tracing.start', {'transferMode': 'ReturnAsStream', 'categories': ','.join(categories),}
        )

    async def stop(self) -> str:
        """Stop tracing.

        :return: trace data as string.
        """
        contentFuture = self._client.loop.create_future()

        async def complete_trace(event):
            nonlocal self, contentFuture
            result = await helpers.readProtocolStream(self._client, event['stream'], self._path)
            contentFuture.set_result(result)

        self._client.once(
            'Tracing.tracingComplete', complete_trace,
        )
        await self._client.send('Tracing.end')
        self._recording = False
        return await contentFuture



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['biolib',
 'biolib.pyppeteer.docs',
 'biolib.pyppeteer.pyppeteer',
 'biolib.pyppeteer.pyppeteer.connection',
 'biolib.pyppeteer.pyppeteer.frame',
 'biolib.pyppeteer.pyppeteer.models',
 'biolib.pyppeteer.utils']

package_data = \
{'': ['*'],
 'biolib': ['biolib-js/*', 'pyppeteer/*', 'pyppeteer/.circleci/*'],
 'biolib.pyppeteer.docs': ['_static/*', '_templates/*']}

install_requires = \
['aenum>=2.2.3,<3.0.0',
 'appdirs>=1.4.3,<2.0.0',
 'certifi>=2019.11.28',
 'nest_asyncio==1.4.0',
 'ordered_set>=4.0.1,<5.0.0',
 'pyee>=7.0.1,<8.0.0',
 'requests==2.24.0',
 'tqdm>=4.42.1,<5.0.0',
 'websockets>=8.1,<9.0']

extras_require = \
{':python_version < "3.8"': ['typing_extensions>=3.7.4,<4.0.0',
                             'typing_inspect>=0.5.0,<0.6.0']}

entry_points = \
{'console_scripts': ['biolib = biolib:cli']}

setup_kwargs = {
    'name': 'pybiolib',
    'version': '0.1.722',
    'description': 'BioLib Python Client',
    'long_description': '# PyBioLib\n\nPyBioLib is a Python package for running BioLib applications from Python scripts, and the command line.\n\n### Python Example\n```python\n# pip3 install pybiolib\nfrom biolib.samtools import samtools\nresult = samtools()\nprint(result.stdout)\n```\n\n### Command Line Example\n```bash\npip3 install pybiolib\nbiolib run samtools/samtools\n```\n',
    'author': 'biolib',
    'author_email': 'hello@biolib.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/biolib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Events module

puppeteer equivalent: Events.js
"""


class Events:
    class Page:
        Close = 'close'
        Console = 'console'
        Dialog = 'dialog'
        DOMContentLoaded = 'domcontentloaded'
        Error = 'error'
        # Can't use just 'error' due to node.js special treatment of error events.
        # @see https://nodejs.org/api/events.html#events_error_events
        PageError = 'pageerror'
        Request = 'request'
        Response = 'response'
        RequestFailed = 'requestfailed'
        RequestFinished = 'requestfinished'
        FrameAttached = 'frameattached'
        FrameDetached = 'framedetached'
        FrameNavigated = 'framenavigated'
        Load = 'load'
        Metrics = 'metrics'
        Popup = 'popup'
        WorkerCreated = 'workercreated'
        WorkerDestroyed = 'workerdestroyed'

    class Browser:
        TargetCreated = 'targetcreated'
        TargetDestroyed = 'targetdestroyed'
        TargetChanged = 'targetchanged'
        Disconnected = 'disconnected'

    class BrowserContext:
        TargetCreated = 'targetcreated'
        TargetDestroyed = 'targetdestroyed'
        TargetChanged = 'targetchanged'

    class NetworkManager:
        Request = 'Events.NetworkManager.Request'
        Response = 'Events.NetworkManager.Response'
        RequestFailed = 'Events.NetworkManager.RequestFailed'
        RequestFinished = 'Events.NetworkManager.RequestFinished'

    class FrameManager:
        FrameAttached = 'Events.FrameManager.FrameAttached'
        FrameNavigated = 'Events.FrameManager.FrameNavigated'
        FrameDetached = 'Events.FrameManager.FrameDetached'
        LifecycleEvent = 'Events.FrameManager.LifecycleEvent'
        FrameNavigatedWithinDocument = 'Events.FrameManager.FrameNavigatedWithinDocument'
        ExecutionContextCreated = 'Events.FrameManager.ExecutionContextCreated'
        ExecutionContextDestroyed = 'Events.FrameManager.ExecutionContextDestroyed'

    class Connection:
        Disconnected = 'Events.Connection.Disconnected'

    class CDPSession:
        Disconnected = 'Events.CDPSession.Disconnected'



import types
import importlib
import sys
import requests
import os
import logging

from biolib.app import BioLibApp, BioLib


def cli():
    # set log level
    env_log_level = os.getenv('BIOLIB_LOG')
    if env_log_level is None:
        BioLib.set_logging(logging.ERROR)
    else:
        env_log_level_upper = env_log_level.upper()
        if env_log_level_upper == "TRACE":
            BioLib.set_logging(BioLib.TRACE_LOGGING)
        elif env_log_level_upper == "DEBUG":
            BioLib.set_logging(logging.DEBUG)
        elif env_log_level_upper == "INFO":
            BioLib.set_logging(logging.INFO)
        elif env_log_level_upper == "WARNING" or env_log_level_upper == "WARN":
            BioLib.set_logging(logging.WARNING)
        else:
            BioLib.set_logging(logging.ERROR)

    if len(sys.argv) > 2 and sys.argv[1] == "run":
        app_splitted = sys.argv[2].split("/")
        if len(app_splitted) == 2:
            app_author = app_splitted[0]
            app_name = app_splitted[1]
            app = BioLibApp(author=app_author, name=app_name)
            stdin = None
            if not sys.stdin.isatty():
                stdin = sys.stdin.read()
            app_args = sys.argv[3:]
            result = app(args=app_args, stdin=stdin, files=None)
            sys.stdout.buffer.write(result.stdout)
            sys.stderr.buffer.write(result.stderr)
        else:
            print(f"App name {sys.argv[2]} was incorrectly formatted. Please use this format: app_developer/app_name")
    else:
        print("Please use the BioLib CLI in the following way: biolib run app_developer/app_name")

class ImportHook(object):

    def find_module(self, fullname, path=None):
        # import hook for all imports in the form blb.*
        if fullname.split('.')[0] == 'biolib':
            return self
        else:
            return None

    def load_module(self, fullname):
        fullname_splitted = fullname.split('.')
        assert fullname_splitted[0] == 'biolib'

        # don't override existing module
        if fullname in sys.modules:
            return sys.modules[fullname]

        # dynamically create new module
        if len(fullname_splitted) == 3:
            sys.modules[fullname] = BioLibApp(author=fullname_splitted[1], name=fullname_splitted[2])
            return sys.modules[fullname]
        elif len(fullname_splitted) == 2:
            spec = importlib.util.spec_from_file_location(fullname, __file__)
            mod = importlib.util.module_from_spec(spec)
            sys.modules[fullname] = mod
        else:
            raise Exception(f'Import `{fullname}` incorrectly formatted')
        return mod

sys.meta_path = [ImportHook()]